import click
import os
import sys
from lrcdl.track import Track, SUPPORTED_EXTENSIONS
from lrcdl.exceptions import (
    UnsupportedExtension,
    LyricsAlreadyExists,
    LyricsNotAvailable,
    TrackNotFound,
    NotEnoughMetadata
)

def error(message, exit=False):
    click.echo(f"{click.style('Error:', fg='red')} {message}")
    if exit:
        sys.exit()

@click.command()
@click.option('--title')
@click.option('--album')
@click.option('--artist')
@click.option('--cache')
@click.option('--recursive', is_flag=True)
@click.option('--include-plain', is_flag=True)
@click.argument('path', type=click.Path())
def lrcdl(path, title, album, artist, cache, recursive, include_plain):
    tracks = []
    skip = []

    if recursive:
        
        if cache and os.path.isdir(cache):
            error("Invalid cache, must be a file", exit=True)
        elif cache and os.path.exists(cache):
            with open(cache, 'r') as cachefile:
                skip = cachefile.read().split('\n')

        if not os.path.isdir(path):
            error("You must specify a directory when --recursive is on", exit=True)
        if title or album or artist:
            error("You cannot specify --title, --album or --artist when --recursive is on", exit=True)
        
        click.echo("Scanning directory...")
        for subdir, _, files in os.walk(path):
            for file in files:
                full_path = os.path.join(subdir, file)
                name, ext = os.path.splitext(full_path)
                if ext.lower() in SUPPORTED_EXTENSIONS and not os.path.exists(name + ".lrc") and full_path not in skip:
                    tracks.append(full_path)
    else:
        tracks.append(path)

    for i, track_path in enumerate(tracks):
        progress = f"({i+1}/{len(tracks)})"
        click.echo(f"Downloading lyrics for {click.style(track_path, bold=True)} {progress}")
        
        try:
            track = Track(track_path, title, album, artist)
            track.download_lyrics(include_plain)
            click.echo(f"Lyrics successfully downloaded for {click.style(track_path, bold=True)}")
        except IsADirectoryError:
            error("A directory was specified when a file path was expected")
        except FileNotFoundError:
            error("Specified path could not be found")
        except UnsupportedExtension:
            error(f"Invalid format, supported extensions are: {', '.join(SUPPORTED_EXTENSIONS)}")
        except LyricsAlreadyExists:
            click.echo(f"Lyrics already exist for {click.style(track_path, bold=True)}. Skipping")
        except NotEnoughMetadata as e:
            click.echo(f"Not enough metadata for {click.style(track_path, bold=True)}. Missing: ({', '.join(e.args[0])}). Specify them manually using --title, --album, and --artist")
        except (LyricsNotAvailable, TrackNotFound):
            skip.append(track_path)
            click.echo(f"Could not find suitable lyrics for {click.style(track_path, bold=True)}")

    if cache:
        with open(cache, 'w') as cachefile:
            cachefile.write('\n'.join(skip))


class TrackNotFound(Exception):
    pass

class LyricsAlreadyExists(Exception):
    pass
 
class LyricsNotAvailable(Exception):
    pass

class UnsupportedExtension(Exception):
    pass

class NotEnoughMetadata(Exception):
    pass


import requests
from importlib.metadata import version
from lrcdl.exceptions import TrackNotFound

DEFAULT_HOST = "https://lrclib.net"
DEFAULT_HEADERS = {
    "User-Agent": f"lrcdl v{version('lrcdl')} (https://github.com/viown/lrcdl)"
}

def get_lyrics(track_name, artist_name, album_name, duration):
    params = {
        "track_name": track_name.strip(),
        "artist_name": artist_name.strip(),
        "album_name": album_name.strip(),
        "duration": duration
    }
    r = requests.get(f"{DEFAULT_HOST}/api/get", params=params, headers=DEFAULT_HEADERS)

    if r.ok:
        return r.json()
    elif r.status_code == 404:
        raise TrackNotFound()


import mutagen
import os
from lrcdl import api
from lrcdl.utils import get_metadata
from lrcdl.exceptions import (
    LyricsAlreadyExists,
    UnsupportedExtension,
    LyricsNotAvailable,
    NotEnoughMetadata
)

SUPPORTED_EXTENSIONS = [".mp3", ".flac"]

class Track:
    def __init__(self, path, title=None, album=None, artist=None):
        self.path = path
        self.split_path = os.path.splitext(self.path)

        if not os.path.exists(self.path):
            raise FileNotFoundError()
        if not os.path.isfile(self.path):
            raise IsADirectoryError()
        if self.split_path[1].lower() not in SUPPORTED_EXTENSIONS:
            raise UnsupportedExtension()
        
        self.file = mutagen.File(self.path)
        
        metadata = get_metadata(self.file)
        self.title = title or metadata["title"]
        self.album = album or metadata["album"]
        self.artist = artist or metadata["artist"]

    def download_lyrics(self, include_plain=False):
        if os.path.exists(self.split_path[0] + ".lrc"):
            raise LyricsAlreadyExists()
        if not (self.title and self.album and self.artist):
            missing = []

            if not self.title:
                missing.append("title")
            if not self.album:
                missing.append("album")
            if not self.artist:
                missing.append("artist")

            raise NotEnoughMetadata(missing)

        lyrics = api.get_lyrics(self.title, self.artist, self.album, round(self.file.info.length))

        lyrics_text = None

        if lyrics["syncedLyrics"]:
            lyrics_text = lyrics["syncedLyrics"]
        elif lyrics["plainLyrics"] and include_plain:
            lyrics_text = lyrics["plainLyrics"]
        else:
            raise LyricsNotAvailable()
        
        with open(self.split_path[0] + ".lrc", "w") as f:
            f.write(lyrics_text)


from mutagen.flac import FLAC
from mutagen.mp3 import MP3

def get_metadata(file):
    metadata = {}
    
    if isinstance(file, MP3):
        metadata["title"] = file.get("TIT2")[0] if "TIT2" in file else None
        metadata["album"] = file.get("TALB")[0] if "TALB" in file else None
        metadata["artist"] = file.get("TPE1")[0] if "TPE1" in file else None
    elif isinstance(file, FLAC):
        metadata["title"] = file.get("title")[0] if "title" in file else None
        metadata["album"] = file.get("album")[0] if "album" in file else None
        metadata["artist"] = file.get("artist")[0] if "artist" in file else None

    return metadata